@@ -1,23 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * e-table-field-chooser.c
- * Copyright (C) 2000  Ximian, Inc.
- * Author: Chris Lahey <clahey@ximian.com>
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -37,6 +38,7 @@ static GtkVBoxClass *parent_class = NULL;
 enum {
 	ARG_0,
 	ARG_FULL_HEADER,
+	ARG_HEADER,
 	ARG_DND_CODE,
 };
 
@@ -85,6 +87,8 @@ e_table_field_chooser_class_init (ETableFieldChooserClass *klass)
 				 GTK_ARG_READWRITE, ARG_DND_CODE);
 	gtk_object_add_arg_type ("ETableFieldChooser::full_header", GTK_TYPE_OBJECT,
 				 GTK_ARG_READWRITE, ARG_FULL_HEADER);
+	gtk_object_add_arg_type ("ETableFieldChooser::header", GTK_TYPE_OBJECT,
+				 GTK_ARG_READWRITE, ARG_HEADER);
 }
 
 static void allocate_callback(GtkWidget *canvas, GtkAllocation *allocation, ETableFieldChooser *etfc)
@@ -155,6 +159,7 @@ e_table_field_chooser_init (ETableFieldChooser *etfc)
 					   e_table_field_chooser_item_get_type(),
 					   "width", (double) 100,
 					   "full_header", etfc->full_header,
+					   "header", etfc->header,
 					   "dnd_code", etfc->dnd_code,
 					   NULL );
 
@@ -184,9 +189,14 @@ e_table_field_chooser_destroy (GtkObject *object)
 	g_free(etfc->dnd_code);
 	if (etfc->full_header)
 		gtk_object_unref(GTK_OBJECT(etfc->full_header));
+	if (etfc->header)
+		gtk_object_unref(GTK_OBJECT(etfc->header));
 
 	if (etfc->gui)
 		gtk_object_unref(GTK_OBJECT(etfc->gui));
+
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 GtkWidget*
@@ -224,6 +234,20 @@ e_table_field_chooser_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 				       "full_header", etfc->full_header,
 				       NULL);
 		break;
+	case ARG_HEADER:
+		if (etfc->header)
+			gtk_object_unref(GTK_OBJECT(etfc->header));
+		if (GTK_VALUE_OBJECT(*arg))
+			etfc->header = E_TABLE_HEADER(GTK_VALUE_OBJECT(*arg));
+		else
+			etfc->header = NULL;
+		if (etfc->header)
+			gtk_object_ref(GTK_OBJECT(etfc->header));
+		if (etfc->item)
+			gtk_object_set(GTK_OBJECT(etfc->item),
+				       "header", etfc->header,
+				       NULL);
+		break;
 	default:
 		break;
 	}
@@ -241,6 +265,9 @@ e_table_field_chooser_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	case ARG_FULL_HEADER:
 		GTK_VALUE_OBJECT (*arg) = GTK_OBJECT(etfc->full_header);
 		break;
+	case ARG_HEADER:
+		GTK_VALUE_OBJECT (*arg) = GTK_OBJECT(etfc->header);
+		break;
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
