@@ -1,28 +1,30 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * e-canvas-vbox.c
- * Copyright (C) 2000  Helix Code, Inc.
- * Author: Chris Lahey <clahey@helixcode.com>
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
-#include <gnome.h>
 #include <math.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtksignal.h>
 #include "e-canvas-vbox.h"
 #include "e-canvas-utils.h"
 #include "e-canvas.h"
@@ -40,6 +42,7 @@ static void e_canvas_vbox_realize (GnomeCanvasItem *item);
 static void e_canvas_vbox_reflow (GnomeCanvasItem *item, int flags);
 
 static void e_canvas_vbox_real_add_item(ECanvasVbox *e_canvas_vbox, GnomeCanvasItem *item);
+static void e_canvas_vbox_real_add_item_start(ECanvasVbox *e_canvas_vbox, GnomeCanvasItem *item);
 static void e_canvas_vbox_resize_children (GnomeCanvasItem *item);
 
 static GnomeCanvasGroupClass *parent_class = NULL;
@@ -48,6 +51,7 @@ static GnomeCanvasGroupClass *parent_class = NULL;
 enum {
 	ARG_0,
 	ARG_WIDTH,
+	ARG_MINIMUM_WIDTH,
 	ARG_HEIGHT,
 	ARG_SPACING,
 };
@@ -90,12 +94,15 @@ e_canvas_vbox_class_init (ECanvasVboxClass *klass)
   
 	gtk_object_add_arg_type ("ECanvasVbox::width", GTK_TYPE_DOUBLE, 
 				 GTK_ARG_READWRITE, ARG_WIDTH); 
+	gtk_object_add_arg_type ("ECanvasVbox::minimum_width", GTK_TYPE_DOUBLE, 
+				 GTK_ARG_READWRITE, ARG_MINIMUM_WIDTH); 
 	gtk_object_add_arg_type ("ECanvasVbox::height", GTK_TYPE_DOUBLE, 
 				 GTK_ARG_READABLE, ARG_HEIGHT);
 	gtk_object_add_arg_type ("ECanvasVbox::spacing", GTK_TYPE_DOUBLE, 
 				 GTK_ARG_READWRITE, ARG_SPACING);
 
 	klass->add_item       = e_canvas_vbox_real_add_item;
+	klass->add_item_start = e_canvas_vbox_real_add_item_start;
  
 	object_class->set_arg   = e_canvas_vbox_set_arg;
 	object_class->get_arg   = e_canvas_vbox_get_arg;
@@ -112,6 +119,7 @@ e_canvas_vbox_init (ECanvasVbox *vbox)
 	vbox->items = NULL;
 
 	vbox->width = 10;
+	vbox->minimum_width = 10;
 	vbox->height = 10;
 	vbox->spacing = 0;
 
@@ -129,7 +137,8 @@ e_canvas_vbox_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	
 	switch (arg_id){
 	case ARG_WIDTH:
-		e_canvas_vbox->width = GTK_VALUE_DOUBLE (*arg);
+	case ARG_MINIMUM_WIDTH:
+		e_canvas_vbox->minimum_width = GTK_VALUE_DOUBLE (*arg);
 		e_canvas_vbox_resize_children(item);
 		e_canvas_item_request_reflow(item);
 		break;
@@ -151,6 +160,9 @@ e_canvas_vbox_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	case ARG_WIDTH:
 		GTK_VALUE_DOUBLE (*arg) = e_canvas_vbox->width;
 		break;
+	case ARG_MINIMUM_WIDTH:
+		GTK_VALUE_DOUBLE (*arg) = e_canvas_vbox->minimum_width;
+		break;
 	case ARG_HEIGHT:
 		GTK_VALUE_DOUBLE (*arg) = e_canvas_vbox->height;
 		break;
@@ -163,12 +175,25 @@ e_canvas_vbox_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	}
 }
 
+/* Used from g_list_foreach(); disconnects from an item's signals */
+static void
+disconnect_item_cb (gpointer data, gpointer user_data)
+{
+	ECanvasVbox *vbox;
+	GnomeCanvasItem *item;
+
+	vbox = E_CANVAS_VBOX (user_data);
+
+	item = GNOME_CANVAS_ITEM (data);
+	gtk_signal_disconnect_by_data (GTK_OBJECT (item), vbox);
+}
+
 static void
 e_canvas_vbox_destroy (GtkObject *object)
 {
 	ECanvasVbox *vbox = E_CANVAS_VBOX(object);
 
-	g_list_foreach(vbox->items, (GFunc) gtk_object_unref, NULL);
+	g_list_foreach(vbox->items, disconnect_item_cb, vbox);
 	g_list_free(vbox->items);
 	vbox->items = NULL;
 	
@@ -225,25 +250,37 @@ e_canvas_vbox_realize (GnomeCanvasItem *item)
 static void
 e_canvas_vbox_remove_item (GnomeCanvasItem *item, ECanvasVbox *vbox)
 {
-	vbox->items = g_list_remove(vbox->items, item);
-	gtk_object_unref(GTK_OBJECT(vbox));
+	if (!GTK_OBJECT_DESTROYED (vbox))
+		vbox->items = g_list_remove(vbox->items, item);
 }
 
 static void
 e_canvas_vbox_real_add_item(ECanvasVbox *e_canvas_vbox, GnomeCanvasItem *item)
 {
 	e_canvas_vbox->items = g_list_append(e_canvas_vbox->items, item);
-	gtk_object_ref(GTK_OBJECT(item));
-	gtk_object_ref(GTK_OBJECT(e_canvas_vbox));
 	gtk_signal_connect(GTK_OBJECT(item), "destroy",
 			   GTK_SIGNAL_FUNC(e_canvas_vbox_remove_item), e_canvas_vbox);
 	if ( GTK_OBJECT_FLAGS( e_canvas_vbox ) & GNOME_CANVAS_ITEM_REALIZED ) {
 		gnome_canvas_item_set(item,
-				      "width", (double) e_canvas_vbox->width,
+				      "width", (double) e_canvas_vbox->minimum_width,
 				      NULL);
 		e_canvas_item_request_reflow(item);
 	}
+}
 
+
+static void
+e_canvas_vbox_real_add_item_start(ECanvasVbox *e_canvas_vbox, GnomeCanvasItem *item)
+{
+	e_canvas_vbox->items = g_list_prepend(e_canvas_vbox->items, item);
+	gtk_signal_connect(GTK_OBJECT(item), "destroy",
+			   GTK_SIGNAL_FUNC(e_canvas_vbox_remove_item), e_canvas_vbox);
+	if ( GTK_OBJECT_FLAGS( e_canvas_vbox ) & GNOME_CANVAS_ITEM_REALIZED ) {
+		gnome_canvas_item_set(item,
+				      "width", (double) e_canvas_vbox->minimum_width,
+				      NULL);
+		e_canvas_item_request_reflow(item);
+	}
 }
 
 static void
@@ -256,7 +293,7 @@ e_canvas_vbox_resize_children (GnomeCanvasItem *item)
 	for ( list = e_canvas_vbox->items; list; list = list->next ) {
 		GnomeCanvasItem *child = GNOME_CANVAS_ITEM(list->data);
 		gnome_canvas_item_set(child,
-				      "width", (double) e_canvas_vbox->width,
+				      "width", (double) e_canvas_vbox->minimum_width,
 				      NULL);
 	}
 }
@@ -269,24 +306,32 @@ e_canvas_vbox_reflow( GnomeCanvasItem *item, int flags )
 
 		gdouble old_height;
 		gdouble running_height;
+		gdouble old_width;
+		gdouble max_width;
+
+		old_width = e_canvas_vbox->width;
+		max_width = e_canvas_vbox->minimum_width;
 
 		old_height = e_canvas_vbox->height;
-		
 		running_height = 0;
 
 		if (e_canvas_vbox->items == NULL) {
 		} else {
 			GList *list;
 			gdouble item_height;
+			gdouble item_width;
 
 			list = e_canvas_vbox->items;
 			gtk_object_get (GTK_OBJECT(list->data),
 					"height", &item_height,
+					"width", &item_width,
 					NULL);
 			e_canvas_item_move_absolute(GNOME_CANVAS_ITEM(list->data),
 						    (double) 0,
 						    (double) running_height);
 			running_height += item_height;
+			if (max_width < item_width)
+				max_width = item_width;
 			list = g_list_next(list);
 			
 			for( ; list; list = g_list_next(list)) {
@@ -294,6 +339,7 @@ e_canvas_vbox_reflow( GnomeCanvasItem *item, int flags )
 
 				gtk_object_get (GTK_OBJECT(list->data),
 						"height", &item_height,
+						"width", &item_width,
 						NULL);
 
 				e_canvas_item_move_absolute(GNOME_CANVAS_ITEM(list->data),
@@ -301,11 +347,15 @@ e_canvas_vbox_reflow( GnomeCanvasItem *item, int flags )
 							    (double) running_height);
 
 				running_height += item_height;
+				if (max_width < item_width)
+					max_width = item_width;
 			}
 				 
 		}
 		e_canvas_vbox->height = running_height;
-		if (old_height != e_canvas_vbox->height)
+		e_canvas_vbox->width = max_width;
+		if (old_height != e_canvas_vbox->height ||
+		    old_width !=  e_canvas_vbox->width)
 			e_canvas_item_request_parent_reflow(item);
 	}
 }
@@ -316,3 +366,11 @@ e_canvas_vbox_add_item(ECanvasVbox *e_canvas_vbox, GnomeCanvasItem *item)
 	if (E_CANVAS_VBOX_CLASS(GTK_OBJECT(e_canvas_vbox)->klass)->add_item)
 		(E_CANVAS_VBOX_CLASS(GTK_OBJECT(e_canvas_vbox)->klass)->add_item) (e_canvas_vbox, item);
 }
+
+void
+e_canvas_vbox_add_item_start(ECanvasVbox *e_canvas_vbox, GnomeCanvasItem *item)
+{
+	if (E_CANVAS_VBOX_CLASS(GTK_OBJECT(e_canvas_vbox)->klass)->add_item_start)
+		(E_CANVAS_VBOX_CLASS(GTK_OBJECT(e_canvas_vbox)->klass)->add_item_start) (e_canvas_vbox, item);
+}
+	
